// nxEdit component consts
const NXEDITPROT_HANDLER_CONTRACTID = "@mozilla.org/network/protocol;1?name=nxedit";
const NXEDITPROT_HANDLER_CID = Components
        .ID("{76cb3a86-6578-9824-8751-67bc3fc28f78}");

// CIDs
const NS_IOSERVICE_CID = "{9ac9e770-18bc-11d3-9337-00104ba0fd40}";
const NS_PREFSERVICE_CONTRACTID = "@mozilla.org/preferences-service;1";
const URI_CONTRACTID = "@mozilla.org/network/simple-uri;1";
const INPUTSTREAMCHANNEL_CONTRACTID = "@mozilla.org/network/input-stream-channel;1";

const CONSOLE_CONTRACTID = "@mozilla.org/consoleservice;1";
// const CONSOLE_INTERFACE = "nsIConsoleService"; !!
const XMLHTTPREQUEST_CID = "@mozilla.org/xmlextras/xmlhttprequest;1";
const NS_LOCALFILE_CID = "@mozilla.org/file/local;1";
const XMLSERIALIZER_CID = "@mozilla.org/xmlextras/xmlserializer;1";

// interfaces
const Ci = Components.interfaces;
const nsIURI = Ci.nsIURI;
const nsIPrefService = Ci.nsIPrefService;
const nsIChannel = Ci.nsIChannel;
const nsIContentPolicy = Ci.nsIContentPolicy;
const nsIConsoleService = Ci.nsIConsoleService;
const nsIXMLHttpRequest = Ci.nsIXMLHttpRequest;
const nsILocalFile = Ci.nsILocalFile;
const nsIDOMSerializer = Ci.nsIDOMSerializer;

const LE_MIME_TYPE = "application/x-nuxeo-liveedit";
const ACCEPT_PREF_BROWSER_KEY = "network.http.accept.default";
const PREF_BRANCH = "extensions.nxedit.";
const NXDEBUG = true;
prefUsername = "Administrator";
prefPassword = "Administrator";
prefWorkdir = "/tmp/";
prefDefaultEditor = "/usr/bin/vi";
prefDefaultEditorArgs = "%s";
prefMapping = "";

function debugLog(msg) {
    if (NXDEBUG) {
        dump(msg + "\n");
        Components.classes[CONSOLE_CONTRACTID].getService(nsIConsoleService)
                .logStringMessage(msg + "\n");
    }
}
/**
 * NxEditProtocolHandler
 */

Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");

function NxEditProtocolHandler()
{
}

NxEditProtocolHandler.prototype = {
  // this must match whatever is in chrome.manifest!
  classID: NXEDITPROT_HANDLER_CID,
  contractId: NXEDITPROT_HANDLER_CONTRACTID,

  QueryInterface: XPCOMUtils.generateQI([Ci.nsIProtocolHandler]),
  scheme : 'nxedit',
  defaultPort : -1,
  protocolFlags : Ci.nsIProtocolHandler.URI_NORELATIVE,
  withinLoad : false,

  /* nsIMyComponent implementation goes here */

  newURI: function(aSpec, aCharset, aBaseURI) {
    var uri = Components.classes[URI_CONTRACTID].createInstance(nsIURI);
    uri.spec = aSpec;
    return uri;
  },

  allowPort : function(aPort, aScheme) {
    return false;
  },

  extractMimeType : function(dom) {
    var mtNode = dom.getElementsByTagName("mimetype").item(0);
    var txtNode = mtNode.firstChild;
    var mimeType = txtNode.nodeValue;
    return mimeType;
  },

  extractMimeType : function(dom) {
    var mtNode = dom.getElementsByTagName("mimetype").item(0);
    var txtNode = mtNode.firstChild;
    var mimeType = txtNode.nodeValue;
    return mimeType;
  },

  getDescriptorId : function(dom) {
    var idNode = dom.getElementsByTagName("editId").item(0);
    var txtNode = idNode.firstChild;
    return txtNode.nodeValue;
  },

  updateUserInfo : function(dom) {
    if (prefPassword != "") {
        debugLog("updateUserInfo : " + prefPassword);
        var passwordNode = dom.getElementsByTagName("userPassword").item(0);
        if (passwordNode == null) {
            debugLog("No passwordNode to upadate!");
            return;
        } else {
            debugLog("PasswordNode found");
            if (passwordNode.firstChild == null) {
                var txtNode = dom.createTextNode(prefPassword);
                passwordNode.appendChild(txtNode);
            } else {
                var txtNode = passwordNode.firstChild;
                txtNode.nodeValue = prefPassword;
            }
        }
    }
    return;
  },

  loadBootstrap : function(aURI) {

    readPreferences(PREF_BRANCH);

    try {
        var PrefService = Components.classes[NS_PREFSERVICE_CONTRACTID]
                .getService(nsIPrefService);
        var prefs = PrefService.getBranch("");
        var isNxWss = prefs.getCharPref(PREF_BRANCH + "isnxwss") == "true" ? true
                : false;
        var bootstrapFileURI = decodeURI(aURI.spec);

        // aURI.spec must have form nxedit:http:// or nxedit:https://
        myURI = decodeURI(aURI.spec).replace("nxedit:", "");
        debugLog("uri=" + myURI);

        if (!isNxWss) {

            var request = Components.classes[XMLHTTPREQUEST_CID]
                    .createInstance();
            request.QueryInterface(nsIXMLHttpRequest);
            try {
                request.open("GET", myURI, false);
                request.send(null);
                debugLog("request sent ok");
            } catch (e) {
                debugLog("Error sending request");
                displayMessage("NXEdit Error", "Can not reach the server");
                return;
            }
            var xmlBootstrap = "";
            if (request.status != 200) {
                debugLog("ResponseKO : " + request.status);
                displayMessage("NXEdit Error",
                        "Can not read bootstrap from server");
                return;
            } else {
                // get the xml content
                var xmlBootstrap = request.responseText;
                var xmlBootstrapDom = request.responseXML;
                debugLog("ResponseOK : " + xmlBootstrap);

                var mimetype = this.extractMimeType(xmlBootstrapDom);
                debugLog("mimetype : " + mimetype);

                this.updateUserInfo(xmlBootstrapDom);

                var bootstrapFilePath = prefWorkdir
                        + this.getDescriptorId(xmlBootstrapDom) + ".xml";
                debugLog("output file : " + bootstrapFilePath);

                // save the xml file
                var file = Components.classes[NS_LOCALFILE_CID]
                        .createInstance(nsILocalFile);
                try {
                    file.initWithPath(bootstrapFilePath);
                } catch (e) {
                    debugLog("error while creating temp file in "
                            + bootstrapFilePath);
                    displayMessage("NXEdit Error",
                            "Can not create temporay resource, please check your settings");
                    return;
                }

                debugLog("file init done");

                var foStream = Components.classes["@mozilla.org/network/file-output-stream;1"]
                        .createInstance(Ci.nsIFileOutputStream);
                foStream.init(file, 0x02 | 0x08 | 0x20, 0666, 0);
                // foStream.write(xmlBootstrap, xmlBootstrap.length);
                // foStream.close();

                var serializer = Components.classes[XMLSERIALIZER_CID]
                        .createInstance();
                serializer.QueryInterface(nsIDOMSerializer);
                // var serializer = new XMLSerializer();
                serializer.serializeToStream(xmlBootstrapDom, foStream, "");
                foStream.close();
                debugLog("file saved");

                debugLog("default editor: " + prefDefaultEditor);
                debugLog("default editor argument: " + prefDefaultEditorArgs);

                // find editor to use
                var mappings = getMapFromStr(prefMapping);
                var editorCmd = prefDefaultEditor;
                var editorCmdArgs = prefDefaultEditorArgs;

                if (prefMapping != "") {
                    // NXP-2545
                    if (mappings.hasKey(mimetype)) {
                        debugLog("mapping found for mimetype " + mimetype);
                        editorCmd = mappings.getCommandValue(mimetype);
                        editorCmdArgs = mappings.getArgValue(mimetype);
                        debugLog("editor found : " + editorCmd + " Args : "
                                + editorCmdArgs);
                    }
                } else {
                    debugLog("using default editor");
                }

                // launch the editor process
                if (editorCmd.length > 0) {
                    debugLog("using editor: " + editorCmd);
                    var editorFile = Components.classes[NS_LOCALFILE_CID]
                            .createInstance(nsILocalFile);
                    editorFile.initWithPath(editorCmd);

                    debugLog("file descriptor for editor created");

                    var process = Components.classes["@mozilla.org/process/util;1"]
                            .createInstance(Ci.nsIProcess);
                    process.init(editorFile);
                    debugLog("file descriptor for editor created");
                    if (editorCmdArgs.match("%s")) {
                        editorCmdArgs = editorCmdArgs.replace("%s",
                                bootstrapFilePath);
                        var args = [ editorCmdArgs, ];
                        debugLog("Arguments (replaced %s) : " + editorCmdArgs);
                    } else {
                        var args = [ editorCmdArgs, bootstrapFilePath, ];
                        debugLog("Arguments (%s not found) : " + editorCmdArgs
                                + " " + bootstrapFilePath);
                    }
                    try {
                        process.run(false, args, args.length);
                    } catch (e) {
                        displayMessage("NXEdit Error",
                                "Error while calling external editor, please check your configuration");
                        debugLog("Error during process exec " + e);
                        return;
                    }
                    debugLog("editor process created!");
                } else {
                    debugLog("no Editor is found");
                    displayMessage("NXEdit Error",
                            "Can not find the associated editor, please check your settings");
                    return;
                }
            }
        }

        else {

            debugLog("NXWSS : uri=" + myURI);

            var mappings = getMapFromStr(prefMapping);
            var editorCmd = prefDefaultEditor;
            var editorCmdArgs = prefDefaultEditorArgs;

            var idx = myURI.indexOf(":");
            if (idx < 0)
                return;

            var mimetype = myURI.substring(0, idx);
            mimetype = mimetype.substring(1, mimetype.length - 1);
            debugLog("Mimetype: " + mimetype);

            myURI = myURI.substring(idx + 1, myURI.length);
            debugLog("myURI: " + myURI);

            debugLog("Mappings: " + mappings);
            if (prefMapping != "") {
                // NXP-2545
                if (mappings.hasKey(mimetype)) {
                    debugLog("mapping found for mimetype " + mimetype);
                    editorCmd = mappings.getCommandValue(mimetype);
                    editorCmdArgs = mappings.getArgValue(mimetype);
                    debugLog("editor found : " + editorCmd + " Args : "
                            + editorCmdArgs);
                }
            } else {
                debugLog("using default editor");
            }

            if (editorCmd.length > 0) {
                debugLog("using editor: " + editorCmd);
                var editorFile = Components.classes[NS_LOCALFILE_CID]
                        .createInstance(nsILocalFile);
                editorFile.initWithPath(editorCmd);

                debugLog("file descriptor for editor created");

                var process = Components.classes["@mozilla.org/process/util;1"]
                        .createInstance(Ci.nsIProcess);
                process.init(editorFile);
                debugLog("file descriptor for editor created");
                if (editorCmdArgs.match("%s")) {
                    editorCmdArgs = editorCmdArgs.replace("%s", myURI);
                    var args = [ editorCmdArgs, ];
                    debugLog("Arguments (replaced %s) : " + editorCmdArgs);
                } else {
                    var args = [ editorCmdArgs, myURI, ];
                    debugLog("Arguments (%s not found) : " + editorCmdArgs
                            + " " + myURI);
                }
                try {
                    process.run(false, args, args.length);
                } catch (e) {
                    displayMessage("NXEdit Error",
                            "Error while calling external editor, please check your configuration");
                    debugLog("Error during process exec " + e);
                    return;
                }
                debugLog("editor process created!");
            } else {
                debugLog("no Editor is found");
                displayMessage("NXEdit Error",
                        "Can not find the associated editor, please check your settings");
                return;
            }

        }
    } catch (e) {
        debugLog("ERROR: " + e);
    }
  },

  shouldLoad : function(ct, cl, org, ctx, mt, ext) {
    return nsIContentPolicy.ACCEPT;
  },

  shouldProcess : function(ct, cl, org, ctx, mt,
        ext) {
    return nsIContentPolicy.ACCEPT;
  },

  newChannel : function(aURI) {
    // load the Bootstrap from Nuxeo5
    var chan = this.loadBootstrap(aURI);

    // return a fake channel
    if (chan == null)
        chan = Components.classes[INPUTSTREAMCHANNEL_CONTRACTID]
                .createInstance(nsIChannel);
    return chan;
  },

  startup : function() {
    debugLog("Starting up NxEditProtocolHandler");
  },

};



/**
 * Moddule entrypoint
 */

// The following line is what XPCOM uses to create components. Each component prototype
// must have a .classID which is used to create it.
const NSGetFactory = XPCOMUtils.generateNSGetFactory([NxEditProtocolHandler]);

// helper function

displayMessage = function(title, message) {
    debugLog("enter displayMessage");
    var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
            .getService(Ci.nsIPromptService);
    var result = promptService.confirm(null, title, message);
}

readPreferences = function(pref_branch) {
    debugLog("Loading preferences on branch : " + pref_branch);
    // get preferences branch
    var PrefService = Components.classes[NS_PREFSERVICE_CONTRACTID]
            .getService(nsIPrefService);
    var myPrefs = PrefService.getBranch("");

    // read preferences (if available)
    if (myPrefs.getPrefType(pref_branch + "username") == myPrefs.PREF_STRING) {
        prefUsername = myPrefs.getCharPref(pref_branch + "username");
        debugLog("loading pref for username : " + prefUsername);
    }
    if (myPrefs.getPrefType(pref_branch + "password") == myPrefs.PREF_STRING) {
        prefPassword = myPrefs.getCharPref(pref_branch + "password");
        debugLog("loading pref for password : " + prefPassword);
    }
    if (myPrefs.getPrefType(pref_branch + "workdir") == myPrefs.PREF_STRING) {
        prefWorkdir = myPrefs.getCharPref(pref_branch + "workdir");
        debugLog("loading pref for workdir : " + prefWorkdir);
    }
    if (myPrefs.getPrefType(pref_branch + "defaulteditor") == myPrefs.PREF_STRING) {
        prefDefaultEditor = myPrefs.getCharPref(pref_branch + "defaulteditor");
        debugLog("loading pref for default editor : " + prefDefaultEditor);
    }
    if (myPrefs.getPrefType(pref_branch + "defaulteditorargs") == myPrefs.PREF_STRING) {
        prefDefaultEditorArgs = myPrefs.getCharPref(pref_branch
                + "defaulteditorargs");
        debugLog("loading pref for default editor arguments : "
                + prefDefaultEditorArgs);
    }
    if (myPrefs.getPrefType(pref_branch + "mimeMapping") == myPrefs.PREF_STRING) {
        prefMapping = myPrefs.getCharPref(pref_branch + "mimeMapping");
        debugLog("loading pref for mapping : " + prefMapping);
    }
}

// duplicated code
// with FF3, we will be able to use
// Components.utils.import("resource://nxeditprotocol/content/maputils.js");
// ...
getMapFromStr = function(mappingStr) {
    var argsSep = "@@args";
    var map = {
        put : function(key, val) {
            this[key] = val;
            if (!this.keyList)
                this.keyList = [];
            if (!this.hasKey(key))
                this.keyList[this.keyList.length] = key;
        },
        get : function(key) {
            return this[key];
        },
        getCommandValue : function(key) {
            var str = this.get(key);
            return str.split(argsSep)[0];
        },
        getArgValue : function(key) {
            var str = this.get(key);
            return str.split(argsSep)[1];
        },
        hasKey : function(key) {
            for (i = 0; i < this.keyList.length; i++) {
                debugLog("looking for *" + key + "* => *" + this.keyList[i]
                        + "*");
                if (this.keyList[i] == key)
                    return true;
            }
            return false;
        }
    }
    var mappings = mappingStr.split(";");
    for (i = 0; mappingItem = mappings[i]; i++) {
        var key = unescape(mappingItem.split(":")[0]);
        var val = unescape(mappingItem.split(":")[1]);
        map.put(key, val);
    }
    return map;
}

debugLog("Nuxeo Live Edit helper loaded");
